/** @file     generic.h
 *  A short description of the source file indicating
 *  its purpose.
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Define to prevent recursive inclusion -------------------------------------
#ifndef __GENERIC_H
#define __GENERIC_H

// Includes ------------------------------------------------------------------
// Exported types ------------------------------------------------------------
typedef unsigned char u8;
typedef unsigned int u16;

// Exported constants --------------------------------------------------------
// CHOOSE ONE CONFIGURATION
#define COM	4
//#define COM 3	
//#define COM	2

#define SEG	32
//#define SEG	24	
//#define SEG	16

#define DEFAULT_CLOCK	CLK_CMSR_LSI
//#define DEFAULT_CLOCK		CLK_CMSR_HSI
#define HSI_PRESC 			CLK_CKDIVR_HSI8

// Exported macro ------------------------------------------------------------
#define rim() {_asm("rim\n");} // enable interrupts 
#define sim() {_asm("sim\n");} // disable interrupts
#define nop() {_asm("nop\n");} // No Operation 
#define trap() {_asm("trap\n");} // Trap (soft IT)
#define halt() {_asm("halt\n");} // HALT

/*-----------------------------------------------------------------------------
Description : Handle the bit from the character variables. 
Comments :    The different parameters of commands are
              - VAR : Name of the character variable where the bit is located.
              - Place : Bit position in the variable (7 6 5 4 3 2 1 0)
              - Value : Can be 0 (reset bit) or not 0 (set bit)
              The "MskBit" command allows to select some bits in a source
              variables and copy it in a destination var (return the value).
              The "ValBit" command returns the value of a bit in a char 
              variable: the bit is reseted if it returns 0 else the bit is set.
              This method generates not an optimised code yet.
-----------------------------------------------------------------------------*/ 
#define SetBit(VAR,Place)         ( VAR |= (1<<Place) )
#define ClrBit(VAR,Place)         ( VAR &= ((1<<Place)^255))
#define ChgBit(VAR,Place)         ( VAR ^= (1<<Place) )
#define AffBit(VAR,Place,Value)   ((Value) ? \
                                   (VAR |= (1<<Place)) : \
                                   (VAR &= ((1<<Place)^255)))
#define MskBit(Dest,Msk,Src)      ( Dest = (Msk & Src) | ((~Msk) & Dest) )

#define ValBit(VAR,Place)         (VAR & (1<<Place))


// Boolean
#define FALSE (0x00)
#define TRUE  !FALSE

// Exported functions ------------------------------------------------------- 

#endif /* __GENERIC_H */





