/** @file     clk.c
 *  Clock Controller settings
**/

/******************** (C) COPYRIGHT 2006 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Includes ------------------------------------------------------------------
#include "clk_reg.h"
#include "generic.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

 // @addtogroup CLK
  
/** All Clock Controller settings have to be put in this function:
** clock master, clock switch if any, HSI prescaler, CCO, clock gating system,..
 * No parameter
*******************************************************************************/
void CLK_Init(void)
	{
		#if DEFAULT_CLOCK==CLK_CMSR_LSI
			CLK_Switch(CLK_SWR_LSI,CLK_ICKR_HSIEN);
		#elif DEFAULT_CLOCK==CLK_CMSR_HSI
			CLK_CKDIVR=HSI_PRESC;
		#endif
		CLK_PCKENR1 = 0x00; // disable peripheral clocks 
		CLK_PCKENR2 = CLK_PCKENR2_AWU;  // disable all peripheral clocks except AWU
   }

/** Allow to switch from one clock to another and to switch off the previous
** one or not.
 * @clock clock to switch ON (value put in CLK_SWR), values available in CLK_reg.h
 * @previous_clock if previous clock has to be kept: write 0 as parameter otherwise
 * take predifined value in CLK_reg.h (value put in CLK_ICKR or CLK_ECKR)
*******************************************************************************/
void CLK_Switch(char clock, char previous_clock)	
	{
	CLK_SWCR|=CLK_SWCR_SWEN;
	CLK_SWR=clock;
	while(!(CLK_SWCR&CLK_SWCR_SWIF));
	if ((previous_clock==CLK_ICKR_LSIEN) | (previous_clock==CLK_ICKR_HSIEN))
		CLK_ICKR&=~previous_clock;			//LSI or HSI
	else
		CLK_ECKR&=~previous_clock;			//LSE or HSE 
}









