/** @file     io_port.c
 *  IOs configuration depending on the number of COM/SEG defined in generic.h
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Includes ------------------------------------------------------------------
#include "io_port_reg.h"
#include "io_port.h"
#include "clk_reg.h"
#include "awu_reg.h"
#include "generic.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------

u8 index=0;
u8 first_CC1=0;
volatile u8 counter_up=FALSE;
volatile u8 counter_down=FALSE;

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

extern u8 LCDRAM[];

 // @addtogroup io_port

/** Initialization of the I/Os (COMs + SEGs) + 2 ext its on PA4/PA5.
 * No parameter
*******************************************************************************/
void IOport_Init(void)
{ 
	#if COM==4
		PCODR =  0x00; 	
    PCDDR |= 0xF0;			// PC4 to PC7 in output chip
    PCCR1 =  0xF0;	
    PCCR2 =  0x00;
	#elif COM==3
		PCODR =  0x00; 	
    PCDDR |= 0x70;	// PC4 to PC6 in outputchip
    PCCR1 =  0x70;	
    PCCR2 =  0x00;
	#elif COM==2
		PCODR =  0x00; 	
    PCDDR |= 0x30;	// PC4, PC5 in output chip
    PCCR1 =  0x30;	
    PCCR2 =  0x00;
	#else 
	#error "wrong COM value defined"
	#endif
    
  #if SEG==32
    // Port B output, push-pull, 0, fast 
    PBODR = 0x00;
    PBDDR = 0xFF;
    PBCR1 = 0xFF;
    PBCR2 = 0x00;
    
    // Port D output, push-pull, 0, fast
    PDODR = 0x00;
    PDDDR = 0xFF;
    PDCR1 = 0xFF;
    PDCR2 = 0x00;
		
  #elif SEG==24
    // Port D output, push-pull, 0, fast
    PDODR = 0x00;
    PDDDR = 0xFF;
    PDCR1 = 0xFF;
    PDCR2 = 0x00;
		
	#elif SEG==16
	#else 
	#error "wrong SEG value defined"
	#endif
    // Port E output, push-pull, 0, fast
    PEODR =  0x00;
    PEDDR = 0xFF;
    PECR1 = 0xFF;
    PECR2 = 0x00;
    
		// Port G  output, push-pull, 0, fast 
    PGODR = 0x00;
    PGDDR = 0xFF;
    PGCR1 = 0xFF;
    PGCR2 = 0x00;
		
		// no contrast control in the AWU method
		// Port A4, A5 ext it
	//	PAIDR = 0x00;
	//	PADDR = 0x00;
	//	PACR1 = 0x30;
	//	PACR2 = 0x30;
				
	//	EXTI_CR1=EXTI_CR1_portA_F;

    // Other ports are not used and remain in reset state 
}

/** Applies once Vdd to segments which have to be ON and Vss to
** segments which have to be OFF with COM line of this phase set to low level
** and other set at Vdd/2, and once voltage inverted to the ones applied
** previously on segments with COM line of this phase set to high level and 
** other set at Vdd/2;
** If clock master is LSI, there is a swith on HSI during the interrupt to speed
** up the processing.
  * No parameter
*******************************************************************************/
void IO_Toggle1(void)
{
	u8 temp;
	u8 ckm_main=DEFAULT_CLOCK;
	
	if (CLK_CMSR==CLK_CMSR_LSI)
	{
		CLK_ICKR |= CLK_ICKR_HSIEN;
		ckm_main=CLK_CMSR_LSI;
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_HSI;
		CLK_CKDIVR=HSI_PRESC;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
	}
	
	#if ((SEG==32) || (SEG==16))
		if (index & (SEG/8))
		{
			if (index & (SEG/4))
				temp=0x80;
			else
				temp=0x20;
		}
		else 
		{
			if (index & (SEG/4))
				temp=0x40;
			else
				temp=0x10;
		}
	#elif SEG==24
		switch (index)
		{
		case 0: temp=0x10; 
						break;
		case 3: temp=0x20; 
						break;
		case 6: temp=0x40; 
						break;
		case 9: temp=0x80;
						break;
		default: break;
		}	
	#endif
	
	if (first_CC1==0)
	{
		#if SEG==32
			PBODR=LCDRAM[index];
			PDODR=LCDRAM[index+1];
			PEODR=LCDRAM[index+2];
			PGODR=LCDRAM[index+3];
		#elif SEG==24
			PDODR=LCDRAM[index];
			PEODR=LCDRAM[index+1];
			PGODR=LCDRAM[index+2];
		#elif SEG==16
			PEODR=LCDRAM[index];
			PGODR=LCDRAM[index+1];
		#endif
		PCCR1=temp;
		#if COM==4
			PCODR=~temp&0xF0;
		#elif COM==3
			PCODR=~temp&0x70;
		#elif COM==2
			PCODR=~temp&0x30;
		#endif
		first_CC1=1;
	}
	else
	{
		#if SEG==32
			PBODR=~LCDRAM[index];
			PDODR=~LCDRAM[++index];
			PEODR=~LCDRAM[++index];
			PGODR=~LCDRAM[++index];
		#elif SEG==24
			PDODR=~LCDRAM[index];
			PEODR=~LCDRAM[++index];
			PGODR=~LCDRAM[++index];
		#elif SEG==16
			PEODR=~LCDRAM[index];
			PGODR=~LCDRAM[++index];
		#endif
		
		PCCR1=temp;
		
		#if COM==4
			PCODR|=0xF0;
		#elif COM==3
			PCODR|=0x70;
		#elif COM==2
			PCODR|=0x30;
		#endif
		
		first_CC1=0;
		index++;
	}
	if (CLK_CMSR!=ckm_main) 
	{ 
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_LSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
		CLK_ICKR&=~CLK_ICKR_HSIEN;			
	}
}

/** To decrease the contrast, COM and SEG lines are set low, to increase it,
** COM lines are once set low, once set high and segments are
** once set high once set low; this is handle through the timing modification
** of the awu interrupt;
** If clock master is LSI, there is a swith on HSI during the interrupt to speed
** up the processing.
  * No parameter
*******************************************************************************/
void IO_Toggle2(void) 
{
	u8 ckm_main=DEFAULT_CLOCK;
	u8 k;
		
	if (CLK_CMSR==CLK_CMSR_LSI)
	{
		CLK_ICKR |= CLK_ICKR_HSIEN;
		ckm_main=CLK_CMSR_LSI;
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_HSI;
		CLK_CKDIVR=HSI_PRESC;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
	}
	if (index==(COM*(SEG/8)))
		index=0;
	#if SEG==32
		PBODR=0;
		PDODR=0;
	#elif SEG==24
		PDODR=0;
	#endif
	PEODR=0;
	PGODR=0;		// all segments at low level (Vrms decrease)
	
	#if COM==4
		PCCR1|=0xF0;	  // COMs as output push pull
		PCODR&=0x0F;		// COMs lines set inactive (low level)
	#elif COM==3

		PCCR1|=0x70;		// COMs as output push pull
		PCODR&=0x8F;		// COMs lines set inactive (low level)
	#elif COM==2
		PCCR1|=0x30;		// COMs as output push pull
		PCODR&=0xCF;		// COMs lines set inactive (low level)
	#endif
	
	if (CLK_CMSR!=ckm_main) 
	{ 
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_LSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
		CLK_ICKR&=~CLK_ICKR_HSIEN;		
	}	

} 

