/** @file     lcd.c
 *  All the LCD functions: LCD RAM clear, LCD ON, LCD Display
 *  its purpose.
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Includes ------------------------------------------------------------------
#include "generic.h"
#include "io_port.h"
#include "awu.h"
#include <string.h>

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------

u8 LCDRAM[COM*(SEG/8)];

/***********************************************************                 
; LCD used contains 36-pins with 32 segment pins and 4 COM pins.
; So, it contains total 32x4 =128 segments.
; LCD contains 8 star-type characters each having 16-segments EACH SO (16 X 8=128)
; One LCD chracter is shown below:
;              A
;      --  ----------
;    X \/  |\H  |I  /|
;         F| \  | J/ |B
;          | G\ | /K |
;          ----- -----
;          |   /| \  |
;        E | L/ |M \N|C
;          | /  |   \|   _
;          -----------  | |DP   
;              D         -

;----------------------------------------------------------
; LCD Pin No   |   Segment    |         Micro Port         |
;                                   QFP80    |    QFP64    |
;----------------------------------------------------------
;  1           |1X,1F,1E,1D   |     PI6      |     PG6      
;  2           |1I,1J,1K,1N   |     PI4      |     PG4      
;  3           |2X,2F,2E,2D   |     PI2      |     PG2      
;  4           |2I,2J,2K,2N   |     PI0      |     PG0      
;  5           |3X,3F,3E,3D   |     PH6      |     PE6      
;  6           |3I,3J,3K,3N   |     PH4      |     PE4      
;  7           |4X,4F,4E,4D   |     PH2      |     PE2      
;  8           |4I,4J,4K,4N   |     PH0      |     PE0      
;  9           |5X,5F,5E,5D   |     PG6      |     PD6      
;  10          |5I,5J,5K,5N   |     PG4      |     PD4      
;  11          |6X,6F,6E,6D   |     PG2      |     PD2      
;  12          |6I,6J,6K,6N   |     PG0      |     PD0      
;  13          |7X,7F,7E,7D   |     PB6      |     PB6      
;  14          |7I,7J,7K,7N   |     PB4      |     PB4      
;  15          |8X,8F,8E,8D   |     PB2      |     PB2      
;  16          |8I,8J,8K,8N   |     PB0      |     PB0      
;  17          |, , ,COM4     |     PD4      |     PC7      
;  18          |, ,COM3,      |     PD3      |     PC6      
;  19          |,COM2,,       |     PD2      |     PC5      
;  20	  	     |COM1,,,       |     PD0      |     PC4      
;  21          |8A,8B,8C,8DP  |     PB1      |     PG7      
;  22          |8H,8G,8L,8M   |     PB3      |     PG5      
;  23          |7A,7B,7C,7DP  |     PB5      |     PG3      
;  24          |7H,7G,7L,7M   |     PB7      |     PG1      
;  25          |6A,6B,6C,6DP  |     PG1      |     PE7      
;  26          |6H,6G,6L,6M   |     PG3      |     PE5      
;  27          |5A,5B,5C,5DP  |     PG5      |     PE3      
;  28          |5H,5G,5L,5M   |     PG7      |     PE1      
;  29          |4A,4B,4C,4DP  |     PH1      |     PD7      
;  30          |4H,4G,4L,4M   |     PH3      |     PD5      
;  31          |3A,3B,3C,3DP  |     PH5      |     PD3      
;  32          |3H,3G,3L,3M   |     PH7      |     PD1      
;  33          |2A,2B,2C,2DP  |     PI1      |     PB7      
;  34          |2H,2G,2L,2M   |     PI3      |     PB5      
;  35          |1A,1B,1C,1DP  |     PI5      |     PB3      
;  36          |1H,1G,1L,1M   |     PI7      |     PB1      
;**********************************************************/
const unsigned int number[10]={0xA665,0x0320,0x2A54,0x2A34,0x0E30,0x2C34,0x2C74,0x2990,0x2E74,0x2E34};


const unsigned int letter[29]={0x2E70,0x323C,0x2444,0x322C,0x2C54,0x2C50,0x2474,0x0E70,0x300C,0x0264,0x1109,0x0444,0x8760,0x8661,0x2664,0x2E50,0x2665,0x2E51,0x2C34,0x3008,0x0664,0x05C0,0x06E1,0x8181,0x8180,0x2184,0x4000,0x0002,0x0000}; // apostroph, dot, space at the end


u8 digit[(SEG/8)];

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

 // @addtogroup LCD

/** Clear the LCD RAM from 0x20 to 0x2f.
 * No parameter
*******************************************************************************/
void Clr_LCDRAM(void)
{
	u8 i;
	for (i=0;i<(COM*(SEG/8));i++)
		LCDRAM[i]=0;
}

/** Switch LCD ON (takes AWU and I/Os ressources),
** enable the interrupts.
 * No parameter
*******************************************************************************/
void LCD_On(void)
{
	IOport_Init();
	AWU_Init();
	rim();				/* authorize interrupts */
}
/** Converts an ascii char to a LCD coding contained in letter or number table.
 * @c character
*******************************************************************************/
void Convert (char* c)
{ 
  u16 car,tmp;
  u8 i;
  const u16 mask[4]={0xF000,0x0F00,0x00F0,0x000F};
  const u8 shift[4]={0xC,0x8,0x4,0x0};

  if ((*c<0x5B)&(*c>0x40))
  car = letter[*c-0x41]; 
  if ((*c<0x3A)&(*c>0x2F)) 
  car = number[*c-0x30];
  if (*c==0x20) car=letter[28];
	if (*c==0x27) car=letter[26];
  if (*c==0x2E) car=letter[27];
  for (i=0;i<(SEG/8);i++)
  {
  	tmp = car&mask[i];
    digit[i] = tmp>>shift[i];
  }
}

/** Writes a char in the LCD frame buffer.
 * @car ascii value of the character
 * @pos LCDRAM index [0:3] where to write the LCD coding value 
*******************************************************************************/
void Write_char(char car,u8 pos)
  {
   u8 i;
   u8 tmp = car;
	 Convert(&tmp);
	 
	 if (pos%2!=0)
	 {
		 for (i=0;i<COM;i++) digit[i]=digit[i]<<4;		// swap nibbles for even LCD digits
		 LCDRAM[((SEG/8)-((pos+1)>>1))] &= 0x0F;
		 LCDRAM[((SEG/4)-((pos+1)>>1))] &= 0x0F;
		#if COM==3
			LCDRAM[((3*SEG/8)-((pos+1)>>1))] &= 0x0F;
		#elif COM==4
			LCDRAM[((3*SEG/8)-((pos+1)>>1))] &= 0x0F;
			LCDRAM[((SEG/2)-((pos+1)>>1))] &= 0x0F;
		#endif
	 }
	 else
	 {
		 LCDRAM[((SEG/8)-((pos+1)>>1))] &= 0xF0;		// clear high or low nibble
		 LCDRAM[((SEG/4)-((pos+1)>>1))] &= 0xF0;
		 #if COM==3
		 LCDRAM[((3*SEG/8)-((pos+1)>>1))] &= 0xF0;
		 #endif
		 #if COM==4
		 LCDRAM[((3*SEG/8)-((pos+1)>>1))] &= 0xF0;
		 LCDRAM[((SEG/2)-((pos+1)>>1))] &= 0xF0;
		 #endif
	 }
		
	LCDRAM[((SEG/8)-((pos+1)>>1))] |= digit[0];				// fill LCDRAM with correct codings
	LCDRAM[((SEG/4)-((pos+1)>>1))] |= digit[1];
	#if SEG==24
		LCDRAM[((3*SEG/8)-((pos+1)>>1))] |= digit[2];
	#endif
	#if SEG==32
		LCDRAM[((3*SEG/8)-((pos+1)>>1))] |= digit[2];
		LCDRAM[((SEG/2)-((pos+1)>>1))] |= digit[3];	 
	#endif
  }
	
/** Allow to display a string of 8 characters on the LCD, this function
** only has to be called from the main.
 * @str string of 8 characters to display
*******************************************************************************/
void LCD_Display(char* str)
{
	u8 i;
  for (i=1;i<(strlen(str)+1);i++) 
		Write_char(*(str+i-1),i); 
}


