/** @file     CLK_reg.h
 *  A short description of the source file indicating
 *  its purpose.
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Define to prevent recursive inclusion -------------------------------------
#ifndef __CLK_REG_H
#define __CLK_REG_H

// Includes ------------------------------------------------------------------
#include "CLK.h"

// Exported types ------------------------------------------------------------
// Exported constants --------------------------------------------------------
// Exported macro ------------------------------------------------------------
// Exported functions ------------------------------------------------------- 

// Internal clock sources control register 
volatile unsigned char CLK_ICKR      @(CLK_base_add);
    #define CLK_ICKR_SHW	     	0x20
		#define CLK_ICKR_LSIRDY    	0x10
    #define CLK_ICKR_LSIEN     	0x08
    #define CLK_ICKR_FHW       	0x04
    #define CLK_ICKR_HSIRDY    	0x02
    #define CLK_ICKR_HSIEN     	0x01
    
// External clock sources control register    
volatile unsigned char CLK_ECKR     @(CLK_base_add+1);
    #define CLK_ECKR_HSERDY   	0x02
    #define CLK_ECKR_HSEEN     	0x01

    
// Clock Master register 
volatile unsigned char CLK_CMSR       @(CLK_base_add+3);
    #define CLK_CMSR_HSI       	0xE1
    #define CLK_CMSR_LSI       	0xD2
    #define CLK_CMSR_HSE       	0xB4

// Clock switch Register 
volatile unsigned char CLK_SWR       @(CLK_base_add+4);
    #define CLK_SWR_HSI       	0xE1
    #define CLK_SWR_LSI       	0xD2
    #define CLK_SWR_HSE       	0xB4

// Clock switch control register 
volatile unsigned char CLK_SWCR  @(CLK_base_add+5);
    #define CLK_SWCR_SWIF      	0x08
    #define CLK_SWCR_SWIE      	0x04
    #define CLK_SWCR_SWEN      	0x02
    #define CLK_SWCR_SWBSY     	0x01

// Clock divisor ratio register 
volatile unsigned char CLK_CKDIVR    @(CLK_base_add+6);
    #define CLK_CKDIVR_HSI1    	0x00		// FHSI = FHSI RC Output
    #define CLK_CKDIVR_HSI2    	0x08		// FHSI = FHSI RC Output/2
    #define CLK_CKDIVR_HSI4    	0x10		// FHSI = FHSI RC Output/4
    #define CLK_CKDIVR_HSI8    	0x18		// FHSI = FHSI RC Output/8
    #define CLK_CKDIVR_CPU1    	0x00		// FCPU = FMASTER
    #define CLK_CKDIVR_CPU2    	0x01		// FCPU = FMASTER/2
    #define CLK_CKDIVR_CPU4    	0x02		// FCPU = FMASTER/4
    #define CLK_CKDIVR_CPU8    	0x03		// FCPU = FMASTER/8
    #define CLK_CKDIVR_CPU16   	0x04		// FCPU = FMASTER/16
    #define CLK_CKDIVR_CPU32   	0x05		// FCPU = FMASTER/32
    #define CLK_CKDIVR_CPU64   	0x06		// FCPU = FMASTER/64
    #define CLK_CKDIVR_CPU128  	0x07		// FCPU = FMASTER/128
    
// Peripheral clock enable 
volatile unsigned char CLK_PCKENR1    @(CLK_base_add+7);
    #define CLK_PCKENR1_0      	0x01
    #define CLK_PCKENR1_1      	0x02
    #define CLK_PCKENR1_2      	0x04
    #define CLK_PCKENR1_3      	0x08
    #define CLK_PCKENR1_4      	0x10
    #define CLK_PCKENR1_5      	0x20
    #define CLK_PCKENR1_6      	0x40
    #define CLK_PCKENR1_7      	0x80
    
// Clock Security System Register     
volatile unsigned char CLK_CSSR      @(CLK_base_add+8);
    #define CLK_CSSR_D         	0x08
    #define CLK_CSSR_DIE       	0x04
    #define CLK_CSSR_AUX       	0x02
    #define CLK_CSSR_EN        	0x01
    
// Clock Output Pin control register 
volatile unsigned char CLK_CCOR      @(CLK_base_add+9);    
    #define CLK_CCOR_DISABLE   	0x00
    #define CLK_CCOR_HSIDIV    	0x01
    #define CLK_CCOR_LSI       	0x03
    #define CLK_CCOR_HSE2      	0x05
    #define CLK_CCOR_CPU       	0x09		// FCPU
    #define CLK_CCOR_CPU2      	0x0B		//FCPU/2
    #define CLK_CCOR_CPU4      	0x0D		//FCPU/4
    #define CLK_CCOR_CPU8      	0x0F		//FCPU/8
    #define CLK_CCOR_CPU16     	0x11		//FCPU/16
    #define CLK_CCOR_CPU32     	0x13		//FCPU/32
    #define CLK_CCOR_CPU64     	0x15		//FCPU/64
    #define CLK_CCOR_HSI       	0x17		 
    #define CLK_CCOR_CKM       	0x19		   
    
// High Speed External Stabilization cycles     
volatile unsigned char CLK_HSESTR   @(CLK_base_add+0xA);
    #define CLK_HSESTR_1       	0xE1		// 1 cycle
    #define CLK_HSESTR_16      	0xD2		// 16 cycles
    #define CLK_HSESTR_256     	0xB4		// 256 cycles
    #define CLK_HSESTR_4096    	0x78		// 4096 cycles
    
// Low Speed External Stabilization cycles     
volatile unsigned char CLK_PCKENR2  @(CLK_base_add+0xB);
    #define CLK_PCKENR2_AWU    	0x04
    #define CLK_PCKENR2_ADC    	0x08
    #define CLK_PCKENR2_CAN    	0x80
    
// High Speed calibration Register     
volatile unsigned char CLK_HSICALR   @(CLK_base_add+0xC);
// Complemented High Speed calibration Register  
volatile unsigned char CLK_NHSICALR   @(CLK_base_add+0xD);
// HSI Clock Calibration Trimming Register  
volatile unsigned char CLK_HSITRIMR   @(CLK_base_add+0xE);

// Swim Register 
volatile unsigned char SWIM_CSR   @(SWIM_base_add);
// Global configuration Register 
volatile unsigned char CFG_GCR   @(CFG_GCR_base_add);

#endif /* __CLK_REG_H */


    
