/** @file     io_port.c
 *  IOs configuration depending on the number of COM/SEG defined in generic.h
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Includes ------------------------------------------------------------------
#include "generic.h"
#include "io_port_reg.h"
#include "io_port.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

extern u8 counter_up;
extern u8 counter_down;

 // @addtogroup io_port

/** IOport_Init: Initialization of the I/Os (COMs + SEGs) + 2 ext its on PA4/PA5.
 * No parameter
*******************************************************************************/
void IOport_Init(void)
{ 
	#if COM==4
		PCODR =  0x00; 		
    PCDDR |= 0xF0;		// chip:PC4 to PC7 in output
    PCCR1 =  0xF0;
    PCCR2 =  0x00;
	#elif COM==3
		PCODR =  0x00; 		
    PCDDR |= 0x70;		// chip:PC4 to PC6 in output
    PCCR1 =  0x70;
    PCCR2 =  0x00;
	#elif COM==2
		PCODR =  0x00; 	
    PCDDR |= 0x30;		// chip: PC4, PC5 in output
    PCCR1 =  0x30;
    PCCR2 =  0x00;
	#else 
	#error "wrong COM value defined"
	#endif
    
  #if SEG==32
    // Port B output, push-pull, 0, fast 
    PBODR = 0x00;
    PBDDR = 0xFF;
    PBCR1 = 0xFF;
    PBCR2 = 0x00;
    
    // Port D output, push-pull, 0, fast
    PDODR = 0x00;
    PDDDR = 0xFF;
    PDCR1 = 0xFF;
    PDCR2 = 0x00;
		
  #elif SEG==24
    // Port D output, push-pull, 0, fast
    PDODR = 0x00;
    PDDDR = 0xFF;
    PDCR1 = 0xFF;
    PDCR2 = 0x00;
		
	#elif SEG==16
	#else 
	#error "wrong SEG value defined"
	#endif
    // Port E output, push-pull, 0, fast
    PEODR = 0x00;
    PEDDR = 0xFF;
    PECR1 = 0xFF;
    PECR2 = 0x00;
		
		// Port G  output, push-pull, 0, fast 
    PGODR = 0x00;
    PGDDR = 0xFF;
    PGCR1 = 0xFF;
    PGCR2 = 0x00;
		
		// Port A4, A5 external interrupt
		PAIDR = 0x00;
		PADDR = 0x00;		// Port A in input
		PACR1 = 0x30;
		PACR2 = 0x30;
				
		EXTI_CR1=EXTI_CR1_portA_F;  
  
	// Other ports are not used and remain in reset state    
}

/** I/Os Interrupt function: used to control the contrast by software 
** (PA4: down, PA5: up).
 * No parameter
*******************************************************************************/

@near @interrupt void ITC_PortA(void)
{
	PACR1=0x00;				//anti bouncing  
	
	if (PAIDR&=0X10)	// Test pin because no status flag for pin interrupt
	{
		counter_up=1;
	}
	else if (PAIDR&=0X20)
	{
		counter_down=1;
	}
}

