/** @file     timer2.c
 *  Timer 2 settings and compare + overflow interrupt functions
**/

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************
*
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STmicroelectronics SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

// Includes ------------------------------------------------------------------
#include "timer2_reg.h"
#include "io_port_reg.h"
#include "clk_reg.h"
#include "generic.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
u8 index=0;
u8 first_CC1=0;
volatile u8 counter_up=FALSE;
volatile u8 counter_down=FALSE;
// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

extern u8 LCDRAM[];

 // @addtogroup timer2
  
/** Configuration of the timer in PWM edge aligned mode to generate 2 RTC 
** (one at compare, one at overflow).
  * No parameter
*******************************************************************************/
void TIM2_Init (void)
{
	if (CLK_CMSR==CLK_CMSR_HSI)
	{
		TIM2_ARRH = OVFH_HSI;  	 
		TIM2_ARRL = OVFL_HSI;
		TIM2_CCR1H = COMPH_HSI;
		TIM2_CCR1L = COMPL_HSI;
		if (CLK_CKDIVR==CLK_CKDIVR_HSI1) TIM2_PSCR = 7;
		if (CLK_CKDIVR==CLK_CKDIVR_HSI2) TIM2_PSCR = 6;
		if (CLK_CKDIVR==CLK_CKDIVR_HSI4) TIM2_PSCR = 5;
		if (CLK_CKDIVR==CLK_CKDIVR_HSI8) TIM2_PSCR = 4;
	}

	if (CLK_CMSR==CLK_CMSR_LSI)
	{
		TIM2_ARRH = OVFH_HSI;   
		TIM2_ARRL = OVFL_LSI;
		TIM2_CCR1H = COMPH_LSI;
		TIM2_CCR1L = COMPL_LSI;
		TIM2_PSCR = 0;
	}

	TIM2_CNTRH = 0;  
	TIM2_CNTRL = 0;  
	TIM2_CCMR1 = TIM2_CCMR1_OCM_PWM2; 
	TIM2_CCER1 = 0; 	
	TIM2_CCER2 = 0; 
	TIM2_IER = TIM2_IER_CC1IE | TIM2_IER_UIE; 
}

/** Launch the counter.
  * No parameter
*******************************************************************************/
void TIM2_Run(void)
{
	TIM2_CR1 = TIM2_CR1_CEN; 
}

/** Timer2 Compare Interrupt function: applies once Vdd to segments
** which have to be ON and Vss to segments which have to be OFF with COM line of 
** this phase set to low level and other set at Vdd/2, and once voltage inverted 
** to the ones applied previously on segments with COM line of this phase set to  
** high level and other set at Vdd/2;
** If clock master is LSI, there is a swith on HSI during the interrupt to speed
** up the processing.
  * No parameter
*******************************************************************************/
@near @interrupt void TIM2_It_CC1(void) 
{
	u8 temp;
	u8 ckm_main=DEFAULT_CLOCK;
	
	if (CLK_CMSR==CLK_CMSR_LSI) //switch to HSI
	{
		CLK_ICKR |= CLK_ICKR_HSIEN;
		TIM2_CR1 = ~TIM2_CR1_CEN;
		ckm_main=CLK_CMSR_LSI;
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_HSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
	}
	
	TIM2_SR1&=~TIM2_SR1_CC1IF;
	
	#if (SEG==32) || (SEG==16)
		if (index & (SEG/8))
		{
			if (index & (SEG/4))
				temp=0x80;
			else
				temp=0x20;
		}
		else 
		{
			if (index & (SEG/4))
				temp=0x40;
			else
				temp=0x10;
		}
	#elif SEG==24
		switch (index)
		{	
		case 0: temp=0x10; 
						break;
		case 3: temp=0x20; 
						break;
		case 6: temp=0x40; 
						break;
		case 9: temp=0x80; 
						break;
		default: break;
		}	
	#endif
	
	if (first_CC1==0)
	{
		#if SEG==32
			PBODR=LCDRAM[index];
			PDODR=LCDRAM[index+1];
			PEODR=LCDRAM[index+2];
			PGODR=LCDRAM[index+3];
		#elif SEG==24
			PDODR=LCDRAM[index];
			PEODR=LCDRAM[index+1];
			PGODR=LCDRAM[index+2];
		#elif SEG==16
			PEODR=LCDRAM[index];
			PGODR=LCDRAM[index+1];
		#endif
		PCCR1=temp;
		#if COM==4
			PCODR=~temp&0xF0;
		#elif COM==3
			PCODR=~temp&0x70;
		#elif COM==2
			PCODR=~temp&0x30;
		#endif
		first_CC1=1;
	}
	else
	{
		#if SEG==32
			PBODR=~LCDRAM[index];
			PDODR=~LCDRAM[++index];
			PEODR=~LCDRAM[++index];
			PGODR=~LCDRAM[++index];
		#elif SEG==24
			PDODR=~LCDRAM[index];
			PEODR=~LCDRAM[++index];
			PGODR=~LCDRAM[++index];
		#elif SEG==16
			PEODR=~LCDRAM[index];
			PGODR=~LCDRAM[++index];
		#endif
		
		PCCR1=temp;
		
		#if COM==4
			PCODR|=0xF0;
		#elif COM==3
			PCODR|=0x70;
		#elif COM==2
			PCODR|=0x30;
		#endif
		
		first_CC1=0;
		index++;
	}
	if (CLK_CMSR!=ckm_main) //switch to main clock
	{ 
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_LSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
		TIM2_CR1 = TIM2_CR1_CEN;
		CLK_ICKR&=~CLK_ICKR_HSIEN;  		
	}	
}

/** Timer2 Overflow Interrupt function: to decrease the contrast, COM 
** and SEG lines are set low, to increase it, COM lines are once set low, once set 
** high and segments are once set high once set low. This is handle through the 
** timing modification of the compare interrupt;
** If clock master is LSI, there is a swith on HSI during the interrupt to speed
** up the processing
  * No parameter
*******************************************************************************/
@near @interrupt void TIM2_It_OVF(void) 
{
	u8 ckm_main=DEFAULT_CLOCK;
		
	if (CLK_CMSR==CLK_CMSR_LSI)	//switch to HSI
	{
		CLK_ICKR |= CLK_ICKR_HSIEN;
		ckm_main=CLK_CMSR_LSI;
		TIM2_CR1 = ~TIM2_CR1_CEN;
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_HSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
	}
	PACR2=0x30;					//anti bouncing
	TIM2_SR1&=~TIM2_SR1_UIF;
	if (index==(COM*(SEG/8)))
	{
		index=0;
		if (counter_up)
		{
			#if DEFAULT_CLOCK==CLK_CMSR_HSI
				if (TIM2_CCR1L < (2*COMPL_HSI)) TIM2_CCR1L +=6;
				counter_up=FALSE;
			#elif DEFAULT_CLOCK==CLK_CMSR_LSI
				if (TIM2_CCR1L < (2*COMPL_LSI)) TIM2_CCR1L +=6;
				counter_up=FALSE;
			#endif
		}
		if (counter_down)
		{
			#if DEFAULT_CLOCK==CLK_CMSR_HSI
				if (TIM2_CCR1L > (COMPL_HSI/13)) TIM2_CCR1L -=6;
				counter_down=FALSE;
			#elif DEFAULT_CLOCK==CLK_CMSR_LSI
				if (TIM2_CCR1L > (COMPL_LSI/13)) TIM2_CCR1L -=6;
				counter_down=FALSE;
			#endif
		}
	}
	// all segments at low level (Vrms decrease)
	#if SEG==32
		PBODR=0;
		PDODR=0;
	#elif SEG==24
		PDODR=0;
	#endif
	PEODR=0;	
	PGODR=0;	
	
	#if COM==4
		PCCR1|=0xF0;	  // COMs as output push pull
		PCODR&=0x0F;		// COMs lines set inactive (low level)
	#elif COM==3
		PCCR1|=0x70;		// COMs as output push pull
		PCODR&=0x8F;		// COMs lines set inactive (low level)
	#elif COM==2	
		PCCR1|=0x30;		// COMs as output push pull
		PCODR&=0xCF;		// COMs lines set inactive (low level)
	#endif
			
	if (CLK_CMSR!=ckm_main) //switch to main clock
	{ 
		CLK_SWCR|=CLK_SWCR_SWEN;
		CLK_SWR=CLK_SWR_LSI;
		while(!(CLK_SWCR&CLK_SWCR_SWIF));
		TIM2_CR1 = TIM2_CR1_CEN;
		CLK_ICKR&=~CLK_ICKR_HSIEN;		
	}	
} 




